<?php
/**
 * Plugin Name: Custom Windows Pinned Tiles
 * Plugin URI: http://celloexpressions.com/plugins/custom-windows-pinned-tiles/
 * Description: Create custom, live-updating tiles (with images!) for Windows 8.1/IE11 users to pin to their Start Screens.
 * Version: 2.0
 * Author: Nick Halsey
 * Author URI: http://celloexpressions.com/
 * Tags: Windows 8, Windows 8.1, Internet Explorer, IE10, IE11, live tile, RSS, App, tiles, start screen, pinned site, branding, favicon
 * License: GPL

=====================================================================================
Copyright (C) 2013 Nick Halsey

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with WordPress; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
=====================================================================================
*/

// Set-up Action and Filter Hooks
if ( is_admin() ){
	register_activation_hook(__FILE__, 'cwptiles_add_defaults');
	register_uninstall_hook(__FILE__, 'cwptiles_delete_plugin_options');
	add_action('admin_init', 'cwptiles_init' );
	add_action('admin_menu', 'cwptiles_add_options_page');
	add_filter( 'plugin_action_links', 'cwptiles_plugin_action_links', 10, 2 );
}

// Delete options table entries ONLY when plugin deactivated AND deleted
function cwptiles_delete_plugin_options() {
	delete_option('cwptiles_options');
}
// Display a Settings link on the main plugins page
function cwptiles_plugin_action_links( $links, $file ) {
	if ( $file == plugin_basename( __FILE__ ) ) {
		$cwptiles_links = '<a href="'.get_admin_url().'options-general.php?page=custom-windows-pinned-tiles%2Fcustom-windows-pinned-tiles.php">'.__('Settings').'</a>';
		// make the 'Settings' link appear first
		array_unshift( $links, $cwptiles_links );
	}
	return $links;
}

// Define default option settings
function cwptiles_add_defaults() {
	$tmp = get_option('cwptiles_options');
    if(!is_array($tmp)) {
		delete_option('cwptiles_options'); 
		$arr = array(
			'plugin_version' => '2.0',
			'notification_frequency' => 360,
			'background-color' => '#111111',
			'imgurl' => plugins_url('/default-wp-logo.png',__FILE__),
			'img_square_310' => '',
			'img_wide_310' => '',
			'img_square_150' => '',
			'img_square_70' => '',
			'title' => get_bloginfo('name')
		);
		update_option('cwptiles_options', $arr);
	}
}

// Init plugin options to white list our options
function cwptiles_init(){
	register_setting( 'cwptiles_plugin_options', 'cwptiles_options', 'cwptiles_validate_options' );
}

// Add menu page
function cwptiles_add_options_page(){
	add_options_page('Custom Windows Pinned Tile Settings', 'Windows Pinned Tiles', 'manage_options', __FILE__, 'cwptiles_render_form');
}

// Prepare the media uploader
function cwptiles_admin_scripts(){
	// must be running 3.5+ to use color pickers and image upload
	wp_enqueue_media();
	wp_enqueue_style( 'wp-color-picker' );
	wp_enqueue_script('pinned-tiles-admin', plugins_url("/pinned-tiles-admin.js", __FILE__), array('wp-color-picker', 'jquery'));
}
if (isset($_GET['page']) && $_GET['page'] == 'custom-windows-pinned-tiles/custom-windows-pinned-tiles.php') {
	add_action('admin_enqueue_scripts', 'cwptiles_admin_scripts');
}
function cwptiles_render_form(){
	$options = get_option('cwptiles_options');
	
	if( !array_key_exists('notification_frequency',$options) ) {
		$options['notification_frequency'] = 360;
		$new_options_2_0 = array( 'img_square_310', 'img_wide_310', 'img_square_150', 'img_square_70' );
		foreach( $new_options_2_0 as $option ) {
			if( !array_key_exists( $option, $options ) )
				$options[$option] = '';
		}
	}	
?>
	<div class="wrap">
	<style type="text/css">
		#tile-preview-container {
			float: right;
			width: 40%;
			min-width: 200px;
			max-width: 410px;
			height: 600px;
			position: relative;
			background: url('<?php echo plugins_url('/start-screen-preview.png',__FILE__); ?>') no-repeat top left;
		}
		.live-preview-tile {
			color: #ffffff;
			width: 118px;
			height: 118px;
			outline: none;
			background-color: <?php echo $options['background-color']; ?>;
			position: absolute;
			overflow: hidden;
			top: 103px;
			left: 93px;
			transition: .2s linear transform, .2s linear -webkit-transform, .2s linear -moz-transform, .2s linear -ms-transform;
		}
		/* Mimic Windows 8 Start Screen behavior for hover and active states */
		.live-preview-tile:hover {
			outline: 3px solid rgba(200,200,200,.15);
		}
		.live-preview-tile:active,
		.live-preview-tile:focus {
			-webkit-transform: perspective(500px) rotateY(20deg);
			-moz-transform: perspective(500px) rotateY(20deg);
			-ms-transform: perspective(500px) rotateY(20deg);
			outline: 3px solid rgba(200,200,200,.15);
		}
		.live-preview-image {
			position: absolute;
			top: 15px;
			left: 28px;
			width: 64px;
			height: 64px;
		}
		.live-preview-image img {
			max-width: 100%;
			max-height: 100%;
		}
		.live-preview-title {
			max-height: 27px;
			position: absolute;
			bottom: 5px;
			left: 15px;
			width: 90px;
			font-family: "Segoe Light", "Segoe UI", Arial, sans-serif;
			font-size: 12px;
			font-weight: 300;
			letter-spacing: -.02em;
			line-height: 13px;
			color: #ffffff;
			overflow: hidden;
			cursor: default;
		}
		#preview-disclaimer {
			position: absolute;
			bottom: 20px;
			left: 20px;
			font-size: 14px;
			text-shadow: 1px 1px 1px #000;
			color: #fff;
		}
		table.form-table {
			clear: none;
			float: left;
			width: 60%;
		}
	</style>
	<h2>Custom Windows Pinned Tile Settings</h2>
	<?php if( cwptiles_getOption( 'error_message', $options ) ) echo '<div class="error">' . cwptiles_getOption( 'error_message', $options ) . '</div>'; ?>
	<form method="post" action="options.php" >	
	<?php settings_fields('cwptiles_plugin_options'); ?>
	<div id="tile-preview-container">
		<div class="live-preview-tile" id="preview-tile">
			<div class="live-preview-image">
				<img alt="Tile Icon" id="tile-img-preview" src="<?php echo $options['imgurl']; ?>" />
			</div>
			<!-- tile icon -->
			<div class="live-preview-title" id="the_title"><?php echo $options['title']; ?></div>
		</div>
		<div id="preview-disclaimer">Note: preview represents view in Windows 8. Windows 8.1 should have a live tile instead, and can use four different tile sizes. Start screen image and preview concept adapted from <a href="http://www.buildmypinnedsite.com/" target="_blank" style="color: #fff">buildmypinnedsite.com</a> (by Microsoft). Live tiles are fed through their system.</div>
    </div>
	<table class="form-table">
		<tr>
			<td colspan="2"><p>
				Your site's tile will automatically live-update and display your most five most recent posts, checking for new updates every 
				<select name="cwptiles_options[notification_frequency]">
					<option value="30" <?php selected( $options['notification_frequency'], 30 ); ?>>30 minutes</option>
					<option value="60" <?php selected( $options['notification_frequency'], 60 ); ?>>hour</option>
					<option value="360" <?php selected( $options['notification_frequency'], 360 ); ?>>6 hours</option>
					<option value="720" <?php selected( $options['notification_frequency'], 720 ); ?>>12 hours</option>
					<option value="1440" <?php selected( $options['notification_frequency'], 1440 ); ?>>day</option>
				</select>
				via your RSS feed. Post titles will typically display, over the background color.
			</p></td>
		</tr>
		<tr>
			<th scope="row">Default Tile Image<br/><em>Will be cropped and resized for the various tile sizes. Use an image with lots of padding for best results, min. size 310x310 px.</em></th>
			<td>
				<input type="url" id="imgurl" name="cwptiles_options[imgurl]" value="<?php echo $options['imgurl']; ?>" onchange="tilepreview_update()" />
				<input type="button" class="button" name="img_upload_button" id="img_upload_button" value="Upload/Select Image" />
				
				<!-- retain the existing resized images -->
				<input type="hidden" name="cwptiles_options[img_square_310]" value="<?php echo $options['img_square_310']; ?>" />
				<input type="hidden" name="cwptiles_options[img_wide_310]" value="<?php echo $options['img_wide_310']; ?>" />
				<input type="hidden" name="cwptiles_options[img_square_150]" value="<?php echo $options['img_square_150']; ?>" />
				<input type="hidden" name="cwptiles_options[img_square_70]" value="<?php echo $options['img_square_70']; ?>" />
			</td>
		</tr>
		<tr>
			<th scope="row">Tile Title</th>
			<td>
				<input type="text" id="title" name="cwptiles_options[title]" value="<?php echo $options['title']; ?>" onchange="tilepreview_update()" />
			</td>
		</tr>
		<tr>
			<th scope="row">Tile Background Color</th>
			<td colspan="2">
				<input type="text" id="color" class="color-field" name="cwptiles_options[background-color]" value="<?php echo $options['background-color']; ?>" onchange="tilepreview_update()"/>
			</td>
		</tr>
		<tr>
			<td>
				<p class="submit">
					<input type="submit" class="button-primary" value="Save Changes" />
				</p>
			</td>
		</tr>
	</table>
</form>
</div>
<?php	
} //options page

// Sanitize and validate input. Accepts an array, return a sanitized array.
function cwptiles_validate_options($input) {
	// sanitize inputs
	$input['title'] = sanitize_text_field( $input['title'] );
	$input['notification_frequency'] = absint( $input['notification_frequency'] );
	
	// don't let users shoot themselves in the foot by trying to set a value other than those MS accepts
	if( ! in_array( $input['notification_frequency'], array( 30, 60, 360, 720, 1440 ) ) )
		$input['notification_frequency'] = 360;

	// Create the various image sizes if the image has been changed.
	// if( $input['imgurl'] != cwptiles_getOption('imgurl') ) { TODO: doesn't account for updating users
		$img = wp_get_image_editor( $input['imgurl'] );
		
		// Make sure the image exists.
		if( !is_wp_error( $img ) ) {
		
			// Save the image as a png in the dedicated cwptiles folder before generating variants. MS requires that the images are PNGs.
			$info = pathinfo( $input['imgurl'] );
			$upload_dir = wp_upload_dir();
			$upload_base_dir = $upload_dir['basedir'];
			$path = trailingslashit( $upload_base_dir ) . 'cwp_tiles/' . $info['filename'];
			$out = $img->save( $path . '.png', 'image/png' );
			
			if( !is_wp_error( $out ) ) {
				
				// create the exact image sizes needed by MS
				$sizes_array = array(
					array ( 'width' => 310, 'height' => 310, 'crop' => true ),
					array ( 'width' => 310, 'height' => 150, 'crop' => true ),
					array ( 'width' => 150, 'height' => 150, 'crop' => true ),
					array ( 'width' => 70, 'height' => 70, 'crop' => true )
				);
				$resize = $img->multi_resize( $sizes_array );
				
				// save the new image URLs in the plugin options
				if ( !is_wp_error( $resize ) ) {
					$base = trailingslashit($upload_dir['baseurl']) . 'cwp_tiles/';
					$input['img_square_310'] = $base . $resize[0]['file'];
					$input['img_wide_310'] = $base . $resize[1]['file'];
					$input['img_square_150'] = $base . $resize[2]['file'];
					$input['img_square_70'] = $base . $resize[3]['file'];
				}
				else
					$input['error_message'] = '<b>Error Generating Images</b>: ' . $resize->get_error_message();
			}
			else
				$input['error_message'] = '<b>Error Converting Image</b>: ' . $out->get_error_message();
		}
		else
			$input['error_message'] = "<b>Error - Image Doesn't Seem to Exist: </b>: " . $img->get_error_message();
	//}
	
	return $input;
}
	
// output
function cwptiles_getOption( $option, $options = null ) {
	if($options == null)
		$options = get_option('cwptiles_options');
	if(array_key_exists($option, $options))
		return $options[$option];
	else
		return false;
}
add_action('wp_head', 'cwptiles_output');
function cwptiles_output() {
	$options = get_option('cwptiles_options');
	
	$feed_url = get_bloginfo('rss2_url');
	
	$polling_uri = 'http://notifications.buildmypinnedsite.com/?feed='.$feed_url.'&amp;id=1;polling-uri2=http://notifications.buildmypinnedsite.com/?feed='.$feed_url.'&amp;id=2;polling-uri3=http://notifications.buildmypinnedsite.com/?feed='.$feed_url.'&amp;id=3;polling-uri4=http://notifications.buildmypinnedsite.com/?feed='.$feed_url.'&amp;id=4;polling-uri5=http://notifications.buildmypinnedsite.com/?feed='.$feed_url.'&amp;id=5';
	if( cwptiles_getOption( 'notification_frequency', $options ) )
		$polling_frequency = cwptiles_getOption( 'notification_frequency', $options );
	else
		$polling_frequency = 720;
	?>
	<!-- For pinned live tiles in Windows 8.1 start screens -->	
	<meta name="application-name" content="<?php echo cwptiles_getOption('title',$options); ?>" />
	<meta name="msapplication-TileColor" content="<?php echo cwptiles_getOption('background-color',$options); ?>" />
	<meta name="msapplication-notification" content="frequency=<?php echo $polling_frequency; ?>;polling-uri=<?php echo $polling_uri; ?>; cycle=1" />
	
	<?php
	// do we have what we need to display the Windows 8.1 image code?
	if( cwptiles_getOption( 'img_square_310', $options ) ) { ?>
		<meta name="msapplication-square310x310logo" content="<?php echo cwptiles_getOption('img_square_310', $options); ?>" />
		<meta name="msapplication-wide310x150logo" content="<?php echo cwptiles_getOption('img_wide_310', $options); ?>" />
		<meta name="msapplication-square150x150logo" content="<?php echo cwptiles_getOption('img_square_150', $options); ?>" />
		<meta name="msapplication-square70x70logo" content="<?php echo cwptiles_getOption('img_square_70', $options); ?>" />
	<?php
	}
	// always do the windows 8 version
	?>
	<meta name="msapplication-TileImage" content="<?php echo cwptiles_getOption('imgurl',$options); ?>" />
	<?php
}